module srisc(clock,reset,pc,npc,instruction,ds,dt,out,zero,ovfl,RegWrite);
parameter 
	L = 8,
	LC = 5;

`define immed_op ALUOp[7]
input clock, reset;
output [31:0] instruction;
output [LC-1:0] npc, pc;

output [L-1:0] ds, dt;
output [L-1:0] out;
output zero, ovfl;
output RegWrite;

wire [4:0] shift;


wire RegDst, ALUSrc, MemtoReg;
wire MemWrite, MemRead;
wire [7:0] ALUOp;
wire [3:0] PCOp;
wire [5:0] func;

wire [5:0] opcode,fn;
wire [4:0] rs,rt,rd;
wire [31:0] immed;
wire [15:0] se;
wire [25:0] jta;

reg [LC-1:0] pc;

instr instr_cache(.address(npc),.inclock(clock),.q(instruction));

assign opcode = instruction[31:26];
assign rs = instruction[25:21];
assign rt = instruction[20:16];
assign rd = instruction[15:11];
assign shift = instruction[10:6];
assign fn = instruction[5:0];
assign jta = instruction[25:0];

assign immed[15:0] = instruction[15:0];
assign se = (instruction[15]? 16'hFFFF: 16'h0000);
assign immed[31:16] = (ALUOp[6]? se: 16'h0000); 

decode_opcode dop(opcode,
	RegDst, ALUSrc, MemtoReg, RegWrite, MemRead, MemWrite,
	jal, ALUOp, PCOp);

wire branch_active;	
branch_control branch_control_unit(
	.control(PCOp[1:0]),
	.branch(branch_active),
	.zero(zero),
	.rt(instruction[16]),
	.ds(ds[L-1])
);
	
wire [1:0] bcontrol;
assign bcontrol = PCOp[3:2];

wire is_branch, branch, jump, jr, jal;
assign jr = RegDst && (fn==6'h08);
assign jump = (bcontrol[1]? bcontrol[0]: 1'b0) || jr;
assign is_branch = (bcontrol[1]? !bcontrol[0]: 1'b0);
assign branch = is_branch && branch_active;


wire [L-1:0] bt;
wire [4:0] rdm;

assign func = (`immed_op? ALUOp[5:0]: (is_branch? 6'h22: fn));
assign bt = (`immed_op? immed[L-1:0] : dt);
assign rdm = (`immed_op? rt : (jal? 5'h1f: rd));
alu #(.L(L)) alu_unit(ds,bt,shift,func,out,zero,ovfl);

wire [31:0] pcr;
wire [L-1:0] dd;
assign pcr = {26'h000000, pc+1};
assign dd = (jal? pcr[L-1:0]: out);

reg_file #(.L(L)) reg_unit(clock,reset,ds,dt,dd,rs,rt,rdm,RegWrite);

wire [31:0] imm;
assign imm = {se,immed[15:0]};

/*
*   if (jump) npc = (jr? ds : jta);
*   else if (branch) npc = pc + imm;
*	else npc  = pc + 1
*/
assign npc = (jump?  (jr? ds[LC-1:0]: jta[LC-1:0]): pc + (branch? imm[LC-1:0]: 1'b1));

always @(posedge clock or negedge reset)
if (!reset) pc <= 5'b00000;
else pc <= npc;


endmodule